@@ -4,7 +4,7 @@ import 'script-loader!../vpython_libraries/glow.min.js';
 import 'script-loader!../vpython_libraries/plotly.min.js';
 import '../style/jquery-ui.custom.css'
 import '../style/ide.css'
-import img from '../vpython_data/flower_texture.jpg'
+import img from '../vpython_data/flower_texture.jpg'     // use this to get the path to vpython_data
 
 // Ensure downstream JupyterLab webpack places the fonts and images in predictable locations
 import '!!file-loader?name=/vpython_data/[name].[ext]!../vpython_data/earth_texture.jpg'
@@ -31,6 +31,7 @@ import '!!file-loader?name=/vpython_data/[name].[ext]!../vpython_data/wood_textu
 export var comm
 var ws = null
 var isopen = false
+var isjupyterlab_vpython = true
 
 console.log('START OF GLOWCOMM')
 
@@ -41,12 +42,7 @@ export function createWebsocket(msg, serviceUrl) {
         var uri = msg.content.data.wsuri
         var url;
     
-        if (document.location.hostname.includes("localhost")){
-           url = "ws://localhost:" + port + uri;
-        }
-        else {
-           url = serviceUrl + port + uri;
-        }
+        url = serviceUrl + port + uri;
         ws = new WebSocket(url);
         ws.binaryType = "arraybuffer";
 		
@@ -84,13 +80,9 @@ export function setupWebsocket(msg,serviceUrl) {
 	wscheckfontsloaded(msg,serviceUrl)
 }
 
-//var datadir = './static/lab/vpython_data/'
-//window.Jupyter_VPython = "./static/lab/vpython_data/" // prefix used by glow.min.js for textures
-
-//var datadir = '/lab/extensions/vpython/static/vpython_data/'
-//window.Jupyter_VPython = "/lab/extensions/vpython/static/vpython_data/" // prefix used by glow.min.js for textures
+// New datadir value for a prebuilt extension (new in JupyterLab 3.0) 
 var url = new URL(img);
-var datadir = url.pathname.substring(0, url.pathname.lastIndexOf('/')) + "/vpython_data/";
+var datadir = url.pathname.substring(0, url.pathname.lastIndexOf('/')) + "/vpython_data/";   // path to vpython_data dir
 window.Jupyter_VPython = datadir // prefix used by glow.min.js for textures
 url = null
 
@@ -203,17 +195,7 @@ function send() { // periodically send events and update_canvas and request obje
 // Should eventually have glowcomm.html, glowcom.js, and glowcommlab.js all import this common component.
 
 window.__GSlang = "vpython"
-box = vp_box
-sphere = vp_sphere
-simple_sphere = vp_simple_sphere
-cylinder = vp_cylinder
-pyramid = vp_pyramid
-cone = vp_cone
-helix = vp_helix
-//ellipsoid = vp_ellipsoid   // This line causes a big problem in jupyterlab vpython extension
-ring = vp_ring
-arrow = vp_arrow
-compound = vp_compound
+
 function msclock() {
     "use strict";
     if (performance.now) return performance.now()
@@ -234,7 +216,7 @@ var lastrange = 1
 var lastautoscale = true
 var lastsliders = {}
 var lastkeysdown = []
-var interval = 33 // milliseconds
+var interval = 17 // milliseconds
 
 function update_canvas() { // mouse location and other stuff
     "use strict";
@@ -339,6 +321,8 @@ function send_pick(cvs, p, seg) {
     "use strict";
     var evt = {event: 'pick', 'canvas': cvs, 'pick': p, 'segment':seg}
 	events.push(evt)
+    if (timer !== null) clearTimeout(timer)
+    send() // send the info NOW
 }
 
 function send_compound(cvs, pos, size, up) {
@@ -346,6 +330,8 @@ function send_compound(cvs, pos, size, up) {
     var evt = {event: '_compound', 'canvas': cvs, 'pos': [pos.x, pos.y, pos.z], 
         'size': [size.x, size.y, size.z], 'up': [up.x, up.y, up.z]}
 	events.push(evt)
+    if (timer !== null) clearTimeout(timer)
+    send() // send the info NOW
 }
 
 var waitfor_canvas = null
@@ -491,7 +477,7 @@ var attrsb = {'a':'userzoom', 'b':'userspin', 'c':'range', 'd':'autoscale', 'e':
               'p':'left', 'q':'right', 'r':'top', 's':'bottom', 't':'_cloneid',
               'u':'logx', 'v':'logy', 'w':'dot', 'x':'dot_radius', 
               'y':'markers', 'z':'legend', 'A':'label','B':'delta', 'C':'marker_color',
-              'D':'size_units', 'E':'userpan', 'F':'scroll'}
+              'D':'size_units', 'E':'userpan', 'F':'scroll', 'G':'choices', 'H':'depth', 'I':'round'}
 
 // methods are X in {'m': '23X....'}
 var methods = {'a':'select', 'b':'pos', 'c':'start', 'd':'stop', 'f':'clear', // unused eghijklmnopvxyzCDFAB
@@ -506,7 +492,7 @@ var vecattrs = ['pos', 'up', 'color', 'trail_color', 'axis', 'size', 'origin', '
                 'marker_color']
                 
 var textattrs = ['text', 'align', 'caption', 'title', 'title_align', 'xtitle', 'ytitle', 'selected', 'capture',
-                 'label', 'append_to_caption', 'append_to_title', 'bind', 'unbind', 'pause', 'GSprint']
+                 'label', 'append_to_caption', 'append_to_title', 'bind', 'unbind', 'pause', 'GSprint', 'choices']
 
 // patt gets idx and attr code; vpatt gets x,y,z of a vector            
 var patt = /(\d+)(.)(.*)/
@@ -550,8 +536,16 @@ function decode(data) {
                     vs = [Number(val[1]), Number(val[2]), Number(val[3]), Number(val[4])]
                 }
 			} else if (textattrs.indexOf(attr) > -1) {
-                // '\n' doesn't survive JSON transmission, so in vpython.py we replace '\n' with '<br>'
-				val = m[3].replace(/<br>/g, "\n")
+                if (attr == 'choices') { // menu choices are wrapped in a list
+                    val = m[3].slice(2,-2)
+                    val = val.replace(/'/g, '') // remove quotes
+                    let s = val.split(',')
+                    val = []
+                    for (let a of s) {val.push(a)}
+                } else {
+                    // '\n' doesn't survive JSON transmission, so in vpython.py we replace '\n' with '<br>'
+                    val = m[3].replace(/<br>/g, "\n")
+                }
 			} else if (attr == 'rotate') { // angle,x,y,z,x,y,z
 				var temp = m[3]
 				val = []
@@ -574,6 +568,9 @@ function decode(data) {
 				}
 			} else if (attr == 'waitfor' || attr == 'pause' || attr == 'delete') {
 				val = m[3]
+            } else if (attr == 'follow') {
+                if (m[3] == 'None') val = null
+                else val = Number(m[3])
 			} else val = Number(m[3])
 			out = {'idx':idx, 'attr':attr, 'val':val}
 			if (datatype == 'attr') as.push(out)
@@ -618,11 +615,20 @@ function o2vec3(p) {
 function handler(data) {
     "use strict";
 	
-	/*
-	console.log('---------------')
-	for (var d in data) {
-		for (var i in data[d]) console.log(i, JSON.stringify(data[d][i]))
-	}
+    /*
+    // Debugging what is sent from server:
+    let found = false
+    for (let d in data) {
+        for (const i in data[d]) {
+            if (!found) {
+                found = true
+                console.log('================')
+            }
+            if (found) {
+                console.log(i, JSON.stringify(data[d][i]))
+            }
+        }
+    }
     */
 	
 	if (data.cmds !== undefined && data.cmds.length > 0) handle_cmds(data.cmds)
@@ -650,7 +656,8 @@ function handle_cmds(dcmds) {
 		//assembling cfg
 		var vlst = ['pos', 'color', 'size', 'axis', 'up', 'direction', 'center', 'forward', 'foreground',
 				 'background', 'ambient', 'linecolor', 'dot_color', 'trail_color', 'textcolor', 'attrval',
-				 'origin', 'normal', 'bumpaxis','texpos', 'start_face_color', 'end_face_color', 'marker_color']
+				 'origin', 'normal', 'bumpaxis','texpos', 'start_face_color', 'end_face_color', 'marker_color',
+				 'start_normal', 'end_normal']
 		if ((obj != 'gcurve') && ( obj != 'gdots' ) ) vlst.push( 'size' )
 		var cfg = {}
 		var objects = []
@@ -664,10 +671,14 @@ function handle_cmds(dcmds) {
 				   cfg[attr] = o2vec3(val)
 				}                            
 			} else if ( (attr == 'pos' && (obj == 'curve' || obj == 'points')) ||
-						(attr == 'path' && obj == 'extrusion') ) { // only occurs in constructor
-				var ptlist = []
-				for (var kk = 0; kk < val.length; kk++) {
-					ptlist.push( o2vec3(val[kk]) )
+						(obj == 'extrusion' && (attr == 'path' || attr == 'color') ) ) { // only occurs in constructor
+				let ptlist = []
+				if (val[0].length === undefined) { // a single triple [x,y,z] to convert to a vector
+					ptlist = o2vec3(val)
+				} else {
+					for (var kk = 0; kk < val.length; kk++) {
+						ptlist.push( o2vec3(val[kk]) )
+					}
 				}
 				cfg[attr] = ptlist
 			} else if (vlst.indexOf(attr) !== -1) {
@@ -803,20 +814,25 @@ function handle_cmds(dcmds) {
 			case 'local_light':   {glowObjs[idx] = local_light(cfg); break}
 			case 'distant_light': {glowObjs[idx] = distant_light(cfg); break}
 			case 'canvas':        {
-				var container = document.getElementById("glowscript");
-				if (container !== null) {
-					window.__context = { glowscript_container: $("#glowscript").removeAttr("id")}
-				}
-				glowObjs[idx] = canvas(cfg)
-				glowObjs[idx]['idx'] = idx
-				try{
-					glowObjs[idx].wrapper[0].addEventListener("contextmenu", function(event){
-						event.preventDefault(); 
-						event.stopPropagation(); 
-					});
-				}
-				catch(err) {
-					console.log("glowcomm canvas contextmenu event : ",err.message);
+				if ((typeof isjupyterlab_vpython !== 'undefined') && (isjupyterlab_vpython === true)) {
+					var container = document.getElementById("glowscript");
+					if (container !== null) {
+						window.__context = { glowscript_container: $("#glowscript").removeAttr("id")}
+					}
+					glowObjs[idx] = canvas(cfg)
+					glowObjs[idx]['idx'] = idx
+					try{
+						glowObjs[idx].wrapper[0].addEventListener("contextmenu", function(event){
+							event.preventDefault(); 
+							event.stopPropagation(); 
+						});
+					}
+					catch(err) {
+						console.log("glowcomm canvas contextmenu event : ",err.message);
+					}
+				} else {
+					glowObjs[idx] = canvas(cfg)
+					glowObjs[idx]['idx'] = idx
 				}
 				break
 					// Display frames per second and render time:
@@ -899,48 +915,6 @@ function handle_cmds(dcmds) {
 			default:
 				console.log("Unable to create object")
 		}
-		
-		/*
-		if (obj === 'redisplay') {
-			var c = document.getElementById(cmd.sceneId)
-			if (c !== null) {
-				var scn = "#" + cmd.sceneId
-				glowObjs[idx].sceneclone = $(scn).clone(true,true)
-				//document.getElementById('glowscript2').appendChild(c)
-				//document.getElementById('glowscript2').replaceWith(c)
-				$('#glowscript2').replaceWith(c)
-				c = document.getElementById(cmd.sceneId)
-				var cont = scn + " .glowscript"
-				window.__context = { glowscript_container:    $(cont) }
-			} else {
-				window.__context = { glowscript_container: $("#glowscript").removeAttr("id") }                    
-				var newcnvs = canvas()
-				for (var obj in glowObjs[idx].objects) {
-					var o = glowObjs[idx].objects[obj]
-					if ((o.constructor.name !== 'curve') && (o.constructor.name !== 'points')) {
-						glowObjs[o.gidx] = o.clone({canvas: newcnvs})
-						var olen = newcnvs.objects.length
-						if (olen > 0) {
-							newcnvs.objects[olen - 1].gidx = o.gidx
-						}
-					}
-				}
-				glowObjs[idx] = newcnvs
-				$("#glowscript2").attr("id",cmd.sceneId)
-			}
-		} else if (obj === 'delete') {
-			b = glowObjs[idx]
-			//console.log("delete : ",idx)
-			if ((b !== null) || (b.visible !== undefined)) {
-				b.visible = false
-			}
-			glowObjs[idx] = null
-		} else if (obj === 'heartbeat') {
-			//console.log("heartbeat")
-		} else if (obj === 'debug') {
-			console.log("debug : ", cmd)
-		}
-		*/
 	} // end of cmds (constructors and special data)
 }
 
@@ -984,9 +958,12 @@ async function handle_methods(dmeth) {
 			}
 			obj.unbind(val, process_binding)
 		} else if (method === "follow") {
-			obj.camera.follow(glowObjs[val])
+            if (val === null) obj.camera.follow(null)
+			else obj.camera.follow(glowObjs[val])
 		} else if (method === "capture") {
-			await obj.capture(val)
+			// val has the form "Tname.png" (display labels) or "Fname.png" (do not display labels)
+			let TF =  (val[0] == 'T') ? true: false
+			await obj.capture(val.slice(1), TF)
 		} else if (method === 'waitfor') {
 			waitfor_canvas = idx
 			waitfor_options = val
@@ -1001,7 +978,7 @@ async function handle_methods(dmeth) {
 			}
 			process_pause()
 		} else if (method === 'pick') {
-			var p = glowObjs[val].mouse.pick()   // wait for pick render; val is canvas
+			var p = glowObjs[val].mouse.pick()  // wait for pick render; val is canvas
 			var seg = null
 			if (p !== null) {
 				if (p instanceof curve) seg = p.segment
